-- JFD_Civs_BohemiaVaclavII_Functions
-- Author: JFD
-- DateCreated: 2/15/2014 6:33:36 PM
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
------------------------------------------------------------------------------------------------------------------------
include("FLuaVector.lua")
include("PlotIterators.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
------------------------------------------------------------------------------------------------------------------------
local g_ConvertTextKey  = Locale.ConvertTextKey
local g_MapGetPlot		= Map.GetPlot
local g_MathCeil		= math.ceil
local g_MathFloor		= math.floor
local g_MathMax			= math.max
local g_MathMin			= math.min
				
local Players 			= Players
local HexToWorld 		= HexToWorld
local ToHexFromGrid 	= ToHexFromGrid
local Teams 			= Teams
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
------------------------------------------------------------------------------------------------------------------------
-- CORE UTILITIES
------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Player.HasTrait then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
----------------------------------------------------------------------------------------------------------------------------
-- MATH UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game.GetRandom
function Game.GetRandom(lower, upper)
	return Game.Rand((upper + 1) - lower, "") + lower
end
local g_GetRandom = Game.GetRandom
------------------------------------------------------------------------------------------------------------------------
-- MOD UTILITIES
------------------------------------------------------------------------------------------------------------------------
--Game_IsCPActive
function Game_IsCPActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "d1b6328c-ff44-4b0d-aad7-c657f83610cd" then
			return true
		end
	end
	return false
end
local g_IsCPActive = (Game_IsCPActive() and Player.HasStateReligion)
------------------------------------------------------------------------------------------------------------------------
--Game_IsVMCActive
function Game_IsVMCActive()
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == "d1b6328c-ff44-4b0d-aad7-c657f83610cd" then
			return true
		end
	end
	return false
end
local g_IsVMCActive = Game_IsVMCActive()
------------------------------------------------------------------------------------------------------------------------
--Sukritact_GetStrongestMilitaryUnit (Sukritact)
function Sukritact_GetStrongestMilitaryUnit(pPlayer, bIgnoreResources, ...)
	local tUnit = {["Combat"] = 0}
	for iKey, sCombatType in pairs(arg) do
		for row in GameInfo.Units("CombatClass = \'" .. sCombatType .. "\'") do
			if pPlayer:CanTrain(row.ID, bIgnoreResources) and row.Combat > tUnit.Combat then
				tUnit = row
			end
		end
	end
	return tUnit.ID
end
--==========================================================================================================================
-- GAME DEFINES
--==========================================================================================================================
------------------------------------------------------------------------------------------------------------------------
local defineMaxMajorCivs = GameDefines["MAX_MAJOR_CIVS"]
local defineMaxMinorCivs = GameDefines["MAX_MINOR_CIVS"]
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
------------------------------------------------------------------------------------------------------------------------
-- GLOBALS
------------------------------------------------------------------------------------------------------------------------
local buildingHradID = GameInfoTypes["BUILDING_JFD_HRAD"]

local traitBohemiaVaclavIIID = GameInfoTypes["TRAIT_JFD_BOHEMIA_VACLAV_II"]

local unitTaborID = GameInfoTypes["UNIT_JFD_TABOR"]
------------------------------------------------------------------------------------------------------------------------
-- FUNCTIONS
------------------------------------------------------------------------------------------------------------------------
--JFD_BohemiaVaclavII_PlayerDoTurn
local buildingDummyBohemiaVaclavIIID = GameInfoTypes["BUILDING_DUMMY_JFD_BOHEMIA_VACLAV_II"]
local buildingDummyBohemiaVaclavIIOtherID = GameInfoTypes["BUILDING_DUMMY_JFD_BOHEMIA_VACLAV_II_OTHER"]
local buildingDummyHradID = GameInfoTypes["BUILDING_DUMMY_JFD_HRAD"]
local buildingDummyTaborID = GameInfoTypes["BUILDING_DUMMY_JFD_TABOR"]
local notificationGreatPersonID = NotificationTypes["NOTIFICATION_GREAT_PERSON_ACTIVE_PLAYER"]
local promotionTaborID = GameInfoTypes["PROMOTION_JFD_TABOR"]
local promotionTaborOtherID = GameInfoTypes["PROMOTION_JFD_TABOR_OTHER"]
local promotionTaborRangedOtherID = GameInfoTypes["PROMOTION_JFD_TABOR_RANGED_OTHER"]
local resourceCowID = GameInfoTypes["RESOURCE_COW"]
local resourceSheepID = GameInfoTypes["RESOURCE_SHEEP"]
local resourceStoneID = GameInfoTypes["RESOURCE_STONE"]
local resourceWheatID = GameInfoTypes["RESOURCE_WHEAT"]
local unitClassInquisitorID = GameInfoTypes["UNITCLASS_INQUISITOR"]
local unitClassMissionaryID = GameInfoTypes["UNITCLASS_MISSIONARY"]
local unitClassProphetID = GameInfoTypes["UNITCLASS_PROPHET"]
function JFD_BohemiaVaclavII_PlayerDoTurn(playerID)
	local player = Players[playerID]
	if (not player:IsAlive()) then return end
	if player:IsBarbarian() then return end
	if player:IsMinorCiv() then return end
	
	--UNIQUE TRAIT
	local playerCapital = player:GetCapitalCity()
	if (not playerCapital) then return end

	if HasTrait(player, traitBohemiaVaclavIIID) then
		for otherPlayerID = 0, defineMaxMajorCivs - 1 do
			local otherPlayer = Players[otherPlayerID]
			if (otherPlayer:IsAlive() and (not otherPlayer:IsPlayerHasOpenBorders(playerID))) then
				for unit in otherPlayer:Units() do
					if otherPlayer:GetID() ~= playerID then
						if unit:GetPlot():GetOwner() == playerID then 
							local numConvertChance = 0
							local unitClassID = unit:GetUnitClassType()
							if (unitClassID == unitClassMissionaryID or unitClassID == unitClassInquisitorID) then
								numConvertChance = 25
							elseif unitClassID == unitClassProphetID then
								numConvertChance = 10
							end
							if (numConvertChance > 0 and g_GetRandom(1,100) <= numConvertChance) then
								local newUnitID = Sukritact_GetStrongestMilitaryUnit(player, false, "UNITCOMBAT_MELEE", "UNITCOMBAT_GUN")
								if newUnitID then
									local plotX, plotY = unit:GetX(), unit:GetY()
									player:InitUnit(newUnitID, plotX, plotY)
									if player:IsHuman() and player:IsTurnActive() then
										local hex = ToHexFromGrid(Vector2(plotX, plotY))
										Events.GameplayAlertMessage(g_ConvertTextKey("TXT_KEY_TRAIT_JFD_BOHEMIA_VACLAV_II_MESSAGE", unit:GetName()))
										Events.GameplayFX(hex.x, hex.y, -1)
									end
									unit:Kill(-1)
								end
							end
						end
					end
				end
			end
		end

		local mainReligionID = player:GetReligionCreatedByPlayer()
		if Player.HasStateReligion then
			if mainReligionID == -1 then
				mainReligionID = player:GetStateReligion()
			end
		end
		if mainReligionID == -1 then
			mainReligionID = player:GetCapitalCity():GetReligiousMajority() 
		end
		if mainReligionID > 0 then
			local numCityStateFollowers = 0
			for otherPlayerID = 0, defineMaxMinorCivs - 1 do
				local otherPlayer = Players[otherPlayerID]
				local otherPlayerCapital = otherPlayer:GetCapitalCity()
				if (otherPlayer:IsAlive() and otherPlayer:IsMinorCiv() and otherPlayerCapital) then
					if (otherPlayerCapital:GetReligiousMajority() == mainReligionID) then
						if (not otherPlayerCapital:IsHasBuilding(buildingDummyBohemiaVaclavIIOtherID)) then
							otherPlayerCapital:SetNumRealBuilding(buildingDummyBohemiaVaclavIIOtherID, 1)
						end
						numCityStateFollowers = numCityStateFollowers + 1
					else
						if otherPlayerCapital:IsHasBuilding(buildingDummyBohemiaVaclavIIOtherID) then
							otherPlayerCapital:SetNumRealBuilding(buildingDummyBohemiaVaclavIIOtherID, 0)
						end
					end
				end
			end
			playerCapital:SetNumRealBuilding(buildingDummyBohemiaVaclavIIID, numCityStateFollowers)
		end
	end

	--UNIQUE BUILDING
	for city in player:Cities() do
		city:SetNumRealBuilding(buildingDummyHradID, 0)
		if city:IsHasBuilding(buildingHradID) then
			if city:IsHasResourceLocal(resourceWheatID) or
			city:IsHasResourceLocal(resourceCowID) or
			city:IsHasResourceLocal(resourceSheepID) or
			city:IsHasResourceLocal(resourceStoneID) then
				city:SetNumRealBuilding(buildingDummyHradID, 1)	
			end
		end
	end

	--UNIQUE UNIT
	local numTabors = 0
	for unit in player:Units() do
		if unit:GetUnitType() == unitTaborID then
			numTabors = numTabors + 1
			break
		end
	end

	if numTabors > 0 then
		for city in player:Cities() do
			local numAdjacentTabors = 0
			local plot = g_MapGetPlot(city:GetX(), city:GetY())
			for adjacentPlot in PlotAreaSweepIterator(plot, 1, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_EXCLUDE) do
				for value = 0,(adjacentPlot:GetNumUnits() - 1) do
					local adjacentUnit = adjacentPlot:GetUnit(value)
					if (adjacentUnit:GetUnitType() == unitTaborID and adjacentUnit:GetOwner() == playerID) then
						numAdjacentTabors = numAdjacentTabors + 1
					end
				end
			end
			city:SetNumRealBuilding(buildingDummyTaborID, numAdjacentTabors)
		end
	
		for unit in player:Units() do
			unit:SetHasPromotion(promotionTaborOtherID, false)
			unit:SetHasPromotion(promotionTaborRangedOtherID, false)
			local unitPlot = unit:GetPlot()
			if unitPlot:GetOwner() == playerID then
				if unit:IsCombatUnit() then
					local isStackedWithTabor = false
					if g_IsCPActive then
						isStackedWithTabor = unit:IsWithinDistanceOfUnitPromotion(promotionTaborID, 0)
					else
						for adjacentPlot in PlotAreaSweepIterator(unitPlot, 1, SECTOR_NORTH, DIRECTION_CLOCKWISE, DIRECTION_OUTWARDS, CENTRE_EXCLUDE) do
							for value = 0,(adjacentPlot:GetNumUnits() - 1) do
								local adjacentUnit = adjacentPlot:GetUnit(value)
								if (adjacentUnit:GetUnitType() == unitTaborID and adjacentUnit:GetOwner() == playerID) then
									isStackedWithTabor = true
									break
								end
							end
						end
					end
					if isStackedWithTabor then
						if unit:IsRanged() then
							unit:SetHasPromotion(promotionTaborRangedOtherID, true)
						else
							unit:SetHasPromotion(promotionTaborOtherID, true)
						end
					end
				end
			end
		end
	end
end
GameEvents.PlayerDoTurn.Add(JFD_BohemiaVaclavII_PlayerDoTurn)
--=======================================================================================================================
--=======================================================================================================================
